import os

from django.conf import settings
from django.core.cache import cache

from naremitcms.classes import NaremitApplication
from .models import DocImport

class DocImportApplication(NaremitApplication):
    # define the markup language to use, current options are:
    #  - 'markdown': Markdown (default)
    #  - 'rest': reStructured Text
    # it is trivial to add other markup languages, just add them to the
    # get_html() function in docimport/management/commands/docimport.py
    MARKUP_LANGUAGE = 'markdown'

    # The template used for the documentation pages
    TEMPLATE = 'naremitcms-docimport/example.html'

    def find_current(self, url, tree):
        for node in tree:
            if node['url'] == url:
                self.node_id = node['id']
                break
            else:
                self.find_current(url, node['children'])

    def get_data_dir(self):
        if not os.path.isdir(self.DATA_DIR):
            raise Exception('Invalid DATA_DIR directory')
        return self.DATA_DIR

    def get_group(self):
        return ('%s.%s' % (
            self.__class__.__module__,
            self.__class__.__name__)
        )[-255:]

    def get_tree(self):
        # get from cache
        cache_key = ('%s_docimporttree' % self.get_group())[-230:]
        tree = cache.get(cache_key)
        if tree is not None:
            return tree

        # not in cache, generate tree
        try:
            root = DocImport.objects.get(group=self.get_group(), level=0)
            tree = self.get_tree_recurse(root)
        except:
            tree = []

        # save to cache
        cache.set(cache_key, tree, 60 * 60 * 24)
        return tree

    def get_tree_path(self):
        path = '/%s' % self.naremit['page']['cached_url']
        if self.naremit['lang']['is_active']:
            path = '/%s%s' % (self.naremit['lang']['code'], path)
        return path

    def get_tree_recurse(self, parent_node):
        nodes = parent_node.get_children().values('id', 'url', 'title', 'has_content')
        for node in nodes:
            node['children'] = self.get_tree_recurse(DocImport.objects.get(id=node['id']))
        return nodes

    def homepage(self):
        self.set_context('docimport', {
            'tree': self.get_tree(),
            'path': self.get_tree_path(),
        })

    def page(self, url):
        self.node_id = None
        tree = self.get_tree()
        self.find_current('%s/' % url, tree)

        # output
        if self.node_id is None:
            self.raise_404()
        else:
            self.set_template(self.TEMPLATE)
            self.set_context('docimport', {
                'tree': tree,
                'path': self.get_tree_path(),
                'html': DocImport.objects.get(id=self.node_id).html
            })

    def urls(self):
        return (
            (r'^$', 'homepage'),
            (r'^(?P<url>(.*))$', 'page'),
        )

    def get_all_urls(self):
        urls = []
        path = '/%s' % self.naremit['page']['cached_url']
        tree = DocImport.objects.filter(group=self.get_group())
        for node in tree:
            if not node.is_root_node():
                urls.append( ('%s%s' % (path, node.url)).replace('//', '/') )

        # handle multilanguage
        if self.naremit['lang']['is_active']:
            multilang_urls = []
            for lang in settings.LANGUAGES:
                for url in urls:
                    multilang_urls.append('/%s%s' % (lang[0], url))
            return multilang_urls
        else:
            return urls

    # unfinished
    def sitemap(self):
        return self.get_all_urls()

    def medusa_urls(self):
        return sorted(self.get_all_urls())



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DocImport',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', models.SlugField()),
                ('url', models.CharField(max_length=255)),
                ('title', models.CharField(max_length=255)),
                ('group', models.CharField(max_length=255)),
                ('html', models.TextField()),
                ('has_content', models.BooleanField(default=True)),
                ('lft', models.PositiveIntegerField(editable=False, db_index=True)),
                ('rght', models.PositiveIntegerField(editable=False, db_index=True)),
                ('tree_id', models.PositiveIntegerField(editable=False, db_index=True)),
                ('level', models.PositiveIntegerField(editable=False, db_index=True)),
                ('parent', mptt.fields.TreeForeignKey(related_name='children', blank=True, to='naremitcms_docimport.DocImport', null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import os
from operator import itemgetter
from importlib import import_module

from django.conf import settings
from django.core.cache import cache
from django.core.management.base import BaseCommand
from django.utils.text import slugify

from naremitcms.models import Application
from naremitcms_docimport.models import DocImport

class Command(BaseCommand):
    def handle(self, *args, **options):
        # create dummy naremit object
        naremit = {
            'lang': { 'code': 'en', 'is_active': settings.USE_I18N },
            'meta': { 'path': '' },
            'page': { 'cached_url': '' },
            'plugins': {}
        }

        # find all docimport apps
        for app in Application.objects.filter(is_active=True):
            module, function = app.function.rsplit('.', 1)
            module = import_module(module)
            function = getattr(module, function)
            for base in function.__bases__:
                if base.__name__ == 'DocImportApplication':
                    app = function(naremit, None, None, None)
                    self.do_import(app.get_group(), app.get_data_dir(), \
                        app.MARKUP_LANGUAGE)

        # clear the cache
        cache.clear()

    def do_import(self, group, datadir, markup_language):
        # delete previous entries for this group
        DocImport.objects.filter(group=group).delete()

        # create dummy root node
        root = DocImport(slug='', title='', group=group, html='')
        root.save()

        # get files
        files = []
        filelist = [ f for f in os.listdir(datadir) ]
        for f in filelist:
            if os.path.isfile(os.path.join(datadir, f)):
                fprefix = os.path.splitext(f)[0]
                fsplit = fprefix.split(' - ')
                if len(fsplit) == 2:
                    numbers = fsplit[0].split('.')
                    files.append({
                        'number': int(numbers.pop()),
                        'parent': '.'.join(numbers),
                        'slug': slugify(fsplit[1].strip()),
                        'title': fsplit[1].strip(),
                        'filename': os.path.join(datadir, f),
                    })

        # sort list
        files = sorted(files, key=itemgetter('parent', 'number'))

        # loop through nodes
        for f in files:
            if f['parent'] == '':
                doc = DocImport(
                    slug=f['slug'],
                    title=f['title'],
                    group=group,
                    html=self.get_html(f['filename'], markup_language),
                    parent=root
                )
                doc.save()
                self.recurse(files, doc, str(f['number']), group, \
                    markup_language)

    def get_html(self, filename, markup_language):
        f = open(filename, 'r')

        # markdown
        if markup_language == 'markdown':
            from markdown import markdown
            return markdown(f.read())

        # reStructured text
        if markup_language == 'rest':
            from docutils.core import publish_parts
            return publish_parts(f.read(), writer_name='html')['html_body']

    def recurse(self, files, parent_node, parent_string, group, markup_language):
        for f in files:
            if f['parent'] == parent_string:
                doc = DocImport(
                    slug=f['slug'],
                    title=f['title'],
                    group=group,
                    html=self.get_html(f['filename'], markup_language),
                    parent=parent_node
                )
                doc.save()
                self.recurse(files, doc, \
                    '%s.%s' % (parent_string, f['number']), group, \
                    markup_language)
